#include "X3D_GLBuilderShapeVisitor.h"
#include "X3D_Appearance.h"
#include "X3D_FillProperties.h"
#include "X3D_LineProperties.h"
#include "X3D_Material.h"
#include "X3D_Shape.h"
#include "GL_Appearance.h"
#include "GL_FillProperties.h"
#include "GL_LineProperties.h"
#include "GL_Material.h"
#include "GL_Shape.h"

namespace X3DTK {
namespace X3D {

GLBuilderShapeVisitor::GLBuilderShapeVisitor()
: ShapeVisitor()
{
  // Enter functions.
  define(Recorder<Appearance>::getEnterFunction(&GLBuilderShapeVisitor::enterAppearance));
  define(Recorder<FillProperties>::getEnterFunction(&GLBuilderShapeVisitor::enterFillProperties));
  define(Recorder<LineProperties>::getEnterFunction(&GLBuilderShapeVisitor::enterLineProperties));
  define(Recorder<Material>::getEnterFunction(&GLBuilderShapeVisitor::enterMaterial));
  define(Recorder<Shape>::getEnterFunction(&GLBuilderShapeVisitor::enterShape));
}

void GLBuilderShapeVisitor::enterAppearance(Appearance *A) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GA = stateVariables->getNode(A);
  if (GA == 0)
  {
    GA = new GL::Appearance();
    GA->setX3DReference(A);
    stateVariables->addCoupleNode(A, GA);
  }   
  stateVariables->pushNode(GA);
}

void GLBuilderShapeVisitor::enterFillProperties(FillProperties *F) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GF = stateVariables->getNode(F);
  if (GF == 0)
  {
    GF = new GL::FillProperties();
    GF->setX3DReference(F);
    stateVariables->addCoupleNode(F, GF);
  }   
  stateVariables->pushNode(GF);
}

void GLBuilderShapeVisitor::enterLineProperties(LineProperties *L) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GL = stateVariables->getNode(L);
  if (GL == 0)
  {
    GL = new GL::LineProperties();
    GL->setX3DReference(L);
    stateVariables->addCoupleNode(L, GL);
  }   
  stateVariables->pushNode(GL);
}

void GLBuilderShapeVisitor::enterMaterial(Material *M) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GM = stateVariables->getNode(M);
  if (GM == 0)
  {
    GM = new GL::Material();
    GM->setX3DReference(M);
    stateVariables->addCoupleNode(M, GM);
  }   
  stateVariables->pushNode(GM);
}

void GLBuilderShapeVisitor::enterShape(Shape *S) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GS = stateVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Shape();
    GS->setX3DReference(S);
    stateVariables->addCoupleNode(S, GS);
  }   
  stateVariables->pushNode(GS);
}

}
}
